﻿-- *****************
-- * Options Panel *
-- *****************

-- declare these BEFORE anything that depends on LFMonitorDB integrity!
function LFMonitor:ResponseOptionsDefault(CreateOptionsAlreadyRunning)

	LFMonitor:ClearAlerts()
	LFMonitorDB = LFMonitor:deepcopy(LFMonitor.defaults)
	LFMonitorCache = {max = 8192}
	LFMonitorWordCache = {max = 8192}
	LFMonitor:SetAlertSkin()
	LFMonitor:ClearAlerts()
	LFMonitor:BuildTables()
	if not CreateOptionsAlreadyRunning then
		LFMonitor:CreateOptions()
		LFMonitor:ChatCommand(nil) -- can't run during CreateOptions because our main panel doesn't exist or needs to be rebuilt
	end

end

function LFMonitor:CheckSettingsVersion(CreateOptionsAlreadyRunning)
	if (not LFMonitorDB.settingsVersion) or LFMonitorDB.settingsVersion < LFMonitor.settingsVersion then
		LFMonitor:ResponseOptionsDefault(CreateOptionsAlreadyRunning)
		message(LFMonitor.LOCALE_STRINGS["Your settings have been reset because they were incompatible with this version of LFMonitor."])
	end
end

local _G = _G
-- local AceGUI = LibStub("AceGUI-3.0")
local bz = LFMonitor.BZ
local bi = LFMonitor.BI
local l = string.lower
local loc = LFMonitor.LOCALE_STRINGS

local function OptionColour(frame, entity)

	local r, g, b = ColorPickerFrame:GetColorRGB()
	local ir, ig, ib, ia

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])
		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(r, g, b, ia)

	else

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])
		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(r, g, b, ia)
		LFMonitor:SetAlertSkin()

	end

	frame:SetTexture(r, g, b, 1)

end

local function OptionOpacity(frame, entity)

	local a = 1 - OpacitySliderFrame:GetValue()
	local ir, ig, ib, ia

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])
		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(ir, ig, ib, a)

	else

		ir, ig, ib, ia = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])
		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(ir, ig, ib, a)
		LFMonitor:SetAlertSkin()
	end

end

local function OptionCancel(frame, entity)

	local r, g, b, a = unpack(ColorPickerFrame.previousValues)

	if ((entity == "timestamp") or (entity == "player") or (entity == "text")) then

		LFMonitorDB.alert.font[entity] = LFMonitor:RGBToHex(r, g, b, a)

	else

		LFMonitorDB.alert.backdrop[entity] = LFMonitor:RGBToHex(r, g, b, a)
		LFMonitor:SetAlertSkin()

	end

	frame:SetTexture(r, g, b, 1)

end

function LFMonitor:ResponseOptionsOkay()

	if(_G.LFMonitorOptionsRespond:IsVisible())then

		LFMonitorDB.responses.alc = _G.LFMonitorAlcEdit:GetText()
		LFMonitorDB.responses.bs = _G.LFMonitorBsEdit:GetText()
		LFMonitorDB.responses.enc = _G.LFMonitorEncEdit:GetText()
		LFMonitorDB.responses.eng = _G.LFMonitorEngEdit:GetText()
		LFMonitorDB.responses.ins = _G.LFMonitorInsEdit:GetText()
		LFMonitorDB.responses.jc = _G.LFMonitorJcEdit:GetText()
		LFMonitorDB.responses.lw = _G.LFMonitorLwEdit:GetText()
		LFMonitorDB.responses.tlr = _G.LFMonitorTlrEdit:GetText()
		LFMonitorDB.responses.lfm = _G.LFMonitorLFGEdit:GetText()

	end

	LFMonitor:BuildTables()

end

function LFMonitor:CustomOptionsOkay()

	local bas = _G.LFMonitorBasicEdit:GetText() or ""
	local adv = _G.LFMonitorAdvEdit:GetText() or ""

	if(_G.LFMonitorOptionsCustom:IsVisible())then

		LFMonitorDB.options.basic = bas

		if (bas ~= "") then

			-- make sure magic characters are escaped to preserve literal values
			-- ^$().[]*+-?%
			bas = string.gsub(bas, "[%^%$%(%)%.%[%]%*%+%-%?%%]", "%%%1")

			--strip multiple spaces
			bas = string.gsub(bas, "%s+", " ")

		end

		LFMonitorDB.options.basicesc = bas
		LFMonitorDB.options.adv = adv

	end

end

local function SetSelectColour(frame, entity)

	local r, g, b, a

	if ((entity == "background") or (entity == "border")) then

		r, g, b, a = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])

	else

		r, g, b, a =  LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])

	end

	frame:SetTexture(r, g, b, 1)

end

function LFMonitor:FontSliderOnValueChanged(self)

	local slider = self:GetName()

	LFMonitorDB.alert.font.height = self:GetValue()
	LFMonitor:SetAlertSkin()
	_G[slider .. "Text"]:SetText(LFMonitorDB.alert.font.height)

end

local function OptAppClick(frame, entity)

	local r, g, b, a, o
	local ir, ig, ib, ia

	if ((entity == "background") or (entity == "border")) then

		r, g, b, a = LFMonitor:HexToRGB(LFMonitorDB.alert.backdrop[entity])
		o = true

	else

		r, g, b, a =  LFMonitor:HexToRGB(LFMonitorDB.alert.font[entity])
		o = false

	end

	ColorPickerFrame.func = function() OptionColour(frame, entity) end
	ColorPickerFrame.opacityFunc = function() OptionOpacity(frame, entity) end
	ColorPickerFrame.cancelFunc = function() OptionCancel(frame, entity) end
	ColorPickerFrame:SetColorRGB(r, g, b)
	ColorPickerFrame.hasOpacity = o
	ColorPickerFrame.opacity = 1 - a
	ColorPickerFrame.previousValues = {r, g, b, a}
	ShowUIPanel(ColorPickerFrame)

end

local function LFMonitor_Sound_DD_OnClick(self, button, down)

	local opt = self.value

	LFMonitor:PlayAlert(opt)
	LFMonitorDB.options.sound = opt
	LFMonitorSoundSelectText:SetText(LFMonitor.sounds[opt].name)

end

local function LFMonitor_Sound_DD_Initialise()

	local info

	for sound = 1, #LFMonitor.sounds do

		info = {}
		info.text = LFMonitor.sounds[sound].name
		info.value = sound
		info.func = LFMonitor_Sound_DD_OnClick
		UIDropDownMenu_AddButton(info)

	end

end

local function LFMonitor_Font_DD_OnClick(self, button, down)

	local opt = self.value

	LFMonitorDB.alert.font.index = opt
	LFMonitor:SetAlertSkin()
	LFMonitorFontSelectText:SetText(LFMonitor.fonts[opt].name)

end

local function LFMonitor_Font_DD_Initialise()

	local info

	for font = 1, #LFMonitor.fonts do

		info = {}
		info.text = LFMonitor.fonts[font].name
		info.value = font
		info.func = LFMonitor_Font_DD_OnClick
		UIDropDownMenu_AddButton(info)

	end

end

local function LFMonitor_TS_DD_OnClick(self, button, down)

	local opt = self.value

	LFMonitorDB.options.timestamp = opt
	LFMonitorTSSelectText:SetText(LFMonitor.timestamps[opt].name)

end

local function LFMonitor_TS_DD_Initialise()

	local info

	for timestamp = 1, #LFMonitor.timestamps do

		info = {}
		info.text = date(LFMonitor.timestamps[timestamp].name) -- use current time as an example
		info.value = timestamp
		info.func = LFMonitor_TS_DD_OnClick
		UIDropDownMenu_AddButton(info)

	end

end

local function AddAceGUIOptionGroup(frame, name)
	frame:SetName(name,"LFMonitor")
	frame:SetCallback("defaults", function() StaticPopup_Show("LFMONITOR_DEFAULTS") end)
	InterfaceOptions_AddCategory(frame.frame)
end
local function CreateMainPanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsMain")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local version = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local author = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")

	title:SetText("LFMonitor")
	title:SetPoint("CENTER", frame, "CENTER", 0, 170)

	version:SetText(_G.GAME_VERSION_LABEL .. " " .. GetAddOnMetadata("LFMonitor", "Version"))
	version:SetPoint("CENTER", frame, "CENTER", 0, 140)

	author:SetText("Authors:\nDeepac\nSeahen (Arcanedamage - Blackrock (US))")
	author:SetPoint("CENTER", frame, "CENTER", 0, 110)

	return frame

end

local function CreateAppearancePanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsAppearance")
	local slider = CreateFrame("Slider", "LFMonitorFontSlider", frame, "OptionsSliderTemplate")
	local tscolour, tsback = LFMonitor:CreateColourBox(frame)
	local pncolour, pnback = LFMonitor:CreateColourBox(frame)
	local mtcolour, mtback = LFMonitor:CreateColourBox(frame)
	local wincolour, winback = LFMonitor:CreateColourBox(frame)
	local borcolour, borback = LFMonitor:CreateColourBox(frame)
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local texttitle = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local wintitle = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local fonttitle = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local tstitle = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local tstext = tscolour:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local pntext = pncolour:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local mttext = mtcolour:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local wintext = wincolour:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local bortext = borcolour:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local fontselect, fontselected = LFMonitor:CreateComboBox(frame, "LFMonitorFontSelect")
	local tsselect, tsselected = LFMonitor:CreateComboBox(frame, "LFMonitorTSSelect")
	local classcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local classtext = classcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")

	local function frameOnShow()

		classcheck:SetChecked(LFMonitorDB.options.useclasscolours)

	end

	frame:SetScript("OnShow", frameOnShow)

	title:SetWidth(400)
	title:SetText(LFMonitor.APPEARANCE_OPTIONS)
	title:SetJustifyH("CENTER")
	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)

	texttitle:SetWidth(400)
	texttitle:SetText(LFMonitor.TEXT)
	texttitle:SetJustifyH("LEFT")
	texttitle:SetPoint("LEFT", title, "TOPLEFT", 42, -40)

	wintitle:SetWidth(400)
	wintitle:SetText(LFMonitor.WINDOW)
	wintitle:SetJustifyH("LEFT")
	wintitle:SetPoint("LEFT", texttitle, "TOPLEFT", 0, -150)

	fonttitle:SetText(LFMonitor.FONT)
	fonttitle:SetJustifyH("LEFT")
	fonttitle:SetPoint("TOPLEFT", texttitle, "TOPLEFT", 200, 0)

	tstitle:SetWidth(400)
	tstitle:SetText(_G.TIMESTAMPS_LABEL)
	tstitle:SetJustifyH("CENTER")
	tstitle:SetPoint("TOPLEFT", frame, 5, -270)

	tscolour:SetPoint("LEFT", texttitle, "TOPLEFT", -27, -35)
	tscolour:SetScript("OnShow", function() SetSelectColour(tsback, "timestamp") end)
	tscolour:SetScript("OnMouseDown", function(self, arg1) if (arg1 == "LeftButton") then OptAppClick(tsback, "timestamp") end end)

	tstext:SetPoint("LEFT", tscolour, "RIGHT", 10, 0)
	tstext:SetTextColor(1, 1, 1, 1)
	tstext:SetText(_G.TIMESTAMPS_LABEL)

	pncolour:SetPoint("LEFT", tscolour, "TOPLEFT", 0, -30)
	pncolour:SetScript("OnShow", function() SetSelectColour(pnback, "player") end)
	pncolour:SetScript("OnMouseDown", function(self, arg1) if (arg1 == "LeftButton") then OptAppClick(pnback, "player") end end)

	pntext:SetPoint("LEFT", pncolour, "RIGHT", 10, 0)
	pntext:SetTextColor(1, 1, 1, 1)
	pntext:SetText(_G.CALENDAR_PLAYER_NAME)

	mtcolour:SetPoint("LEFT", pncolour, "TOPLEFT", 0, -30)
	mtcolour:SetScript("OnShow", function() SetSelectColour(mtback, "text") end)
	mtcolour:SetScript("OnMouseDown", function(self, arg1) if (arg1 == "LeftButton") then OptAppClick(mtback, "text") end end)

	mttext:SetPoint("LEFT", mtcolour, "RIGHT", 10, 0)
	mttext:SetTextColor(1, 1, 1, 1)
	mttext:SetText(LFMonitor.MESSAGE_TEXT)

	classcheck:SetPoint("LEFT", mtcolour, "TOPLEFT", -4, -45)
	classcheck:SetScript("OnClick", function() LFMonitorDB.options.useclasscolours = classcheck:GetChecked() or 0 end)
	classtext:SetPoint("LEFT", classcheck, "RIGHT", 4, 0)
	classtext:SetTextColor(1, 1, 1, 1)
	classtext:SetText(LFMonitor.CLASS_COLOUR_TEXT)

	wincolour:SetPoint("LEFT", wintitle, "TOPLEFT", -27, -35)
	wincolour:SetScript("OnShow", function() SetSelectColour(winback, "background") end)
	wincolour:SetScript("OnMouseDown", function(self, arg1) if (arg1 == "LeftButton") then OptAppClick(mtback, "background") end end)

	wintext:SetPoint("LEFT", wincolour, "RIGHT", 10, 0)
	wintext:SetTextColor(1, 1, 1, 1)
	wintext:SetText(LFMonitor.WINDOW)

	borcolour:SetPoint("LEFT", wincolour, "TOPLEFT", 0, -30)
	borcolour:SetScript("OnShow", function() SetSelectColour(borback, "border") end)
	borcolour:SetScript("OnMouseDown", function(self, arg1) if (arg1 == "LeftButton") then OptAppClick(borback, "border") end end)

	bortext:SetPoint("LEFT", borcolour, "RIGHT", 10, 0)
	bortext:SetTextColor(1, 1, 1, 1)
	bortext:SetText(_G.EMBLEM_BORDER)

	local function fontOnShow()

		fontselected:SetText(LFMonitor.fonts[LFMonitorDB.alert.font.index].name)
		UIDropDownMenu_Initialize(fontselect, LFMonitor_Font_DD_Initialise)

	end

	fontselect:SetPoint("CENTER", fonttitle, "CENTER", 0, -30)
	fontselect:SetScript("OnShow", fontOnShow)

	slider:SetPoint("CENTER", fontselect, "CENTER", 0, -40)
	slider:SetMinMaxValues(6, 18)
	slider:SetValueStep(1)

	local function sliderOnShow()

		slider:SetValue(LFMonitorDB.alert.font.height)
		LFMonitorFontSliderLow:SetText(6)
		LFMonitorFontSliderHigh:SetText(18)
		LFMonitorFontSliderText:SetText(LFMonitorDB.alert.font.height)

	end

	slider:SetScript("OnShow", sliderOnShow)

	local function sliderOnValueChanged()

		LFMonitorDB.alert.font.height = slider:GetValue()
		LFMonitor:SetAlertSkin()
		LFMonitorFontSliderText:SetText(LFMonitorDB.alert.font.height)

	end

	slider:SetScript("OnValueChanged", sliderOnValueChanged)

	local function tsOnShow()

		tsselected:SetText(LFMonitor.timestamps[LFMonitorDB.options.timestamp].name)
		UIDropDownMenu_Initialize(tsselect, LFMonitor_TS_DD_Initialise)

	end

	tsselect:SetPoint("CENTER", tstitle, "CENTER", 0, -35)
	tsselect:SetScript("OnShow", tsOnShow)

	return frame

end

local function CreateGeneralPanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsAppearance")
	local discheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local lockcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local showframe = CreateFrame("Frame", nil, frame)
	local restcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local tradecheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local combcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local instcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local soundcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local generalcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local tradechancheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local guildcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local lfgcheck = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	local sensitivityslider = CreateFrame("Slider", nil, frame, "OptionsSliderTemplate")
	local cacheslider = CreateFrame("Slider", nil, frame, "OptionsSliderTemplate")
	local showtex = showframe:CreateTexture()
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local channels = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local distext = discheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local cltext = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local locktext = lockcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local resttext = restcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local tradetext = restcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local combtext = combcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local insttext = instcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local soundtext = soundcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local generaltext = generalcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local tradechantext = tradecheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local guildtext = guildcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local lfgtext = guildcheck:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local soundselect, soundselected = LFMonitor:CreateComboBox(frame, "LFMonitorSoundSelect")
	local sensitivitytext = sensitivityslider:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local cachetext = cacheslider:CreateFontString(nil, "ARTWORK", "GameFontNormal")

	local function GeneralOnShow()

		discheck:SetChecked(LFMonitorDB.options.disabled)
		lockcheck:SetChecked(LFMonitorDB.options.locked)
		instcheck:SetChecked(LFMonitorDB.options.hinst)
		restcheck:SetChecked(LFMonitorDB.options.rested)
		tradecheck:SetChecked(LFMonitorDB.options.trade)
		combcheck:SetChecked(LFMonitorDB.options.combat)
		soundcheck:SetChecked(LFMonitorDB.options.soundon)
		generalcheck:SetChecked(LFMonitorDB.channels.general)
		tradechancheck:SetChecked(LFMonitorDB.channels.trade)
		guildcheck:SetChecked(LFMonitorDB.channels.guild)
		lfgcheck:SetChecked(LFMonitorDB.channels.lfg)

		if (soundcheck:GetChecked()) then

			soundselect:Show()

		else

			soundselect:Hide()

		end

	end

	frame:SetScript("OnShow", GeneralOnShow)

	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)
	title:SetJustifyH("CENTER")
	title:SetWidth(400)
	title:SetText(LFMonitor.GENERAL_OPTIONS)

	channels:SetPoint("TOPLEFT", title, "TOPLEFT", 42, -240)
	channels:SetWidth(400)
	channels:SetJustifyH("CENTER")
	channels:SetText(LFMonitor.CHANNELS)

	showframe:SetPoint("TOPLEFT", title, "TOPLEFT", 6, -22)
	showframe:SetHeight(62)
	showframe:SetWidth(390)
	showframe:SetBackdrop({edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16})
	showframe:SetBackdropBorderColor(0.6, 0.6, 0.6, 0.6)
	showtex:SetAllPoints(showframe)

	local function radioOnClick(chk1, chk2)

		if ((chk1:GetChecked()) and (chk2:GetChecked())) then

			chk2:SetChecked(0)

		end

		LFMonitorDB.options.rested = restcheck:GetChecked()
		LFMonitorDB.options.trade = tradecheck:GetChecked()
		LFMonitor:DisableCheck()

	end

	restcheck:SetPoint("LEFT", title, "TOPLEFT", 15, -40)
	restcheck:SetScript("OnClick", function() radioOnClick(restcheck, tradecheck) end)

	resttext:SetPoint("LEFT", restcheck, "RIGHT")
	resttext:SetTextColor(1, 1, 1, 1)
	resttext:SetText(LFMonitor.SHOW_IN_CITIES)

	tradecheck:SetPoint("LEFT", restcheck, "TOPLEFT", 0, -40)
	tradecheck:SetScript("OnClick", function() radioOnClick(tradecheck, restcheck) end)

	tradetext:SetPoint("LEFT", tradecheck, "RIGHT")
	tradetext:SetTextColor(1, 1, 1, 1)
	tradetext:SetText(LFMonitor.TRADE_CHANNEL)

	discheck:SetPoint("LEFT", tradecheck, "TOPLEFT", 0, -45)
	discheck:SetScript("OnClick", function() LFMonitorDB.options.disabled = discheck:GetChecked(); LFMonitor:DisableCheck(discheck:GetChecked()) end)

	distext:SetPoint("LEFT", discheck, "RIGHT")
	distext:SetTextColor(1, 1, 1, 1)
	distext:SetText(_G.ADDON_DISABLED)

	lockcheck:SetPoint("LEFT", discheck, "TOPLEFT", 0, -40)
	lockcheck:SetScript("OnClick", function() LFMonitor:SetLocked(lockcheck:GetChecked()) end)

	locktext:SetPoint("LEFT", lockcheck, "RIGHT")
	locktext:SetTextColor(1, 1, 1, 1)
	locktext:SetText(_G.LOCK_FRAME)

	combcheck:SetPoint("LEFT", lockcheck, "TOPLEFT", 0, -40)
	combcheck:SetScript("OnClick", function() LFMonitorDB.options.combat = combcheck:GetChecked() end)

	combtext:SetPoint("LEFT", combcheck, "RIGHT")
	combtext:SetTextColor(1, 1, 1, 1)
	combtext:SetText(LFMonitor.HIDE_IN_COMBAT)

	instcheck:SetPoint("LEFT", combcheck, "TOPLEFT", 0, -40)
	instcheck:SetScript("OnClick", function() LFMonitorDB.options.hinst = instcheck:GetChecked(); LFMonitor:DisableCheck(); end)

	insttext:SetPoint("LEFT", instcheck, "RIGHT")
	insttext:SetTextColor(1, 1, 1, 1)
	insttext:SetText(LFMonitor.HIDE_IN_INSTANCES)

	soundcheck:SetPoint("LEFT", instcheck, "TOPLEFT", 0, -40)
	soundcheck:SetScript("OnClick", function() LFMonitorDB.options.soundon = soundcheck:GetChecked(); if (soundcheck:GetChecked()) then soundselect:Show() else soundselect:Hide() end end)

	soundtext:SetPoint("LEFT", soundcheck, "RIGHT")
	soundtext:SetTextColor(1, 1, 1, 1)
	soundtext:SetText(_G.ENABLE_SOUND)

	local function soundOnShow()

		soundselected:SetText(LFMonitor.sounds[LFMonitorDB.options.sound].name)
		UIDropDownMenu_Initialize(soundselect, LFMonitor_Sound_DD_Initialise)

	end

	soundselect:SetPoint("LEFT", soundcheck, "RIGHT", 100, -2)
	soundselect:SetScript("OnShow", soundOnShow)

	generalcheck:SetPoint("LEFT", soundcheck, "TOPLEFT", 0, -80)
	generalcheck:SetScript("OnClick", function() LFMonitorDB.channels.general = generalcheck:GetChecked() end)

	generaltext:SetPoint("LEFT", generalcheck, "RIGHT")
	generaltext:SetTextColor(1, 1, 1, 1)
	generaltext:SetText(_G.GENERAL)

	tradechancheck:SetPoint("LEFT", generalcheck, "TOPLEFT", 0, -40)
	tradechancheck:SetScript("OnClick", function() LFMonitorDB.channels.trade = tradechancheck:GetChecked() end)

	tradechantext:SetPoint("LEFT", tradechancheck, "RIGHT")
	tradechantext:SetTextColor(1, 1, 1, 1)
	tradechantext:SetText(_G.TRADE)

	guildcheck:SetPoint("LEFT", tradechancheck, "TOPLEFT", 0, -40)
	guildcheck:SetScript("OnClick", function() LFMonitorDB.channels.guild = guildcheck:GetChecked() end)

	guildtext:SetPoint("LEFT", guildcheck, "RIGHT")
	guildtext:SetTextColor(1, 1, 1, 1)
	guildtext:SetText(_G.GUILD)

	lfgcheck:SetPoint("TOPLEFT", generalcheck, "TOPLEFT", 180, 0)
	lfgcheck:SetScript("OnClick", function() LFMonitorDB.channels.lfg = lfgcheck:GetChecked() end)

	lfgtext:SetPoint("LEFT", lfgcheck, "RIGHT")
	lfgtext:SetTextColor(1, 1, 1, 1)
	lfgtext:SetText(_G.LFG_TITLE)

	cachetext:SetPoint("TOPLEFT", guildcheck, "TOPLEFT", 0, -40)
	cachetext:SetTextColor(1, 1, 1, 1)
	cachetext:SetText(LFMonitor.CACHE_SIZE)

	cacheslider:SetPoint("TOPLEFT", cachetext, "BOTTOMLEFT", 0, 0)
	cacheslider:SetWidth(180)
	cacheslider:SetOrientation("HORIZONTAL")
	cacheslider:SetMinMaxValues(10,20)
	cacheslider:SetValueStep(1)

	local function cacheSliderOnShow()
		cacheslider:SetValue(math.log(LFMonitorCache.max)/math.log(2))
	end
	cacheslider:SetScript("OnShow",cacheSliderOnShow)

	local function cacheSliderOnValueChanged()
		LFMonitorCache = {count=0, max = 2^cacheslider:GetValue()}
		LFMonitorWordCache = {count=0, max = 2^cacheslider:GetValue()}
	end
	cacheslider:SetScript("OnValueChanged",cacheSliderOnValueChanged)

	sensitivitytext:SetPoint("TOPLEFT", cachetext, "TOPLEFT", 200, 0)
	sensitivitytext:SetTextColor(1, 1, 1, 1)
	sensitivitytext:SetText(LFMonitor.SENSITIVITY)

	sensitivityslider:SetPoint("TOPLEFT", sensitivitytext, "BOTTOMLEFT", 0, 0)
	sensitivityslider:SetWidth(180)
	sensitivityslider:SetOrientation("HORIZONTAL")
	sensitivityslider:SetMinMaxValues(0,.9)
	sensitivityslider:SetValueStep(.1)
	sensitivityslider:SetScript("OnShow", function() sensitivityslider:SetValue(LFMonitorDB.options.sensitivity) end)
	sensitivityslider:SetScript("OnValueChanged", function() LFMonitorDB.options.sensitivity = sensitivityslider:GetValue() end)
	return frame
end

local function CreateResponsePanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsRespond")
	local alcedit = CreateFrame("EditBox", "LFMonitorAlcEdit", frame, "InputBoxTemplate")
	local bsedit = CreateFrame("EditBox", "LFMonitorBsEdit", frame, "InputBoxTemplate")
	local encedit = CreateFrame("EditBox", "LFMonitorEncEdit", frame, "InputBoxTemplate")
	local engedit = CreateFrame("EditBox", "LFMonitorEngEdit", frame, "InputBoxTemplate")
	local insedit = CreateFrame("EditBox", "LFMonitorInsEdit", frame, "InputBoxTemplate")
	local jcedit = CreateFrame("EditBox", "LFMonitorJcEdit", frame, "InputBoxTemplate")
	local lwedit = CreateFrame("EditBox", "LFMonitorLwEdit", frame, "InputBoxTemplate")
	local tlredit = CreateFrame("EditBox", "LFMonitorTlrEdit", frame, "InputBoxTemplate")
	local lfgedit = CreateFrame("EditBox", "LFMonitorLFGEdit", frame, "InputBoxTemplate")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local alctext = alcedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local bstext = bsedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local enctext = encedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local engtext = engedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local instext = insedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local jctext = jcedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local lwtext = lwedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local tlrtext = tlredit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local lfgtext = lfgedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local edits = {
		{box = alcedit, text = alctext, db = "alc", title = LFMonitor.ALCHEMY},
		{box = bsedit, text = bstext, db = "bs", title = LFMonitor.BLACKSMITHING},
		{box = encedit, text = enctext, db = "enc", title = LFMonitor.ENCHANTING},
		{box = engedit, text = engtext, db = "eng", title = LFMonitor.ENGINEERING},
		{box = insedit, text = instext, db = "ins", title = LFMonitor.INSCRIPTION},
		{box = jcedit, text = jctext, db = "jc", title = LFMonitor.JEWELCRAFTING},
		{box = lwedit, text = lwtext, db = "lw", title = LFMonitor.LEATHERWORKING},
		{box = tlredit, text = tlrtext, db = "tlr", title = LFMonitor.TAILORING},
		}
	local first, anchor = true, title
	local offsety, offsetx = -55, 10

	frame:Hide()

	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)
	title:SetWidth(400)
	title:SetText(LFMonitor.RESPONSES_TO_REQUESTS)

	local function responseOnShow()

		for _, edit in ipairs(edits) do

			if first then

				anchor = title
				first = nil
				offsetx = 10

			else

				offsetx = 0

			end

				edit.box:SetPoint("LEFT", anchor, "TOPLEFT", offsetx, offsety)
				edit.box:SetWidth(385)
				edit.box:SetHeight(32)
				edit.box:SetMaxLetters(90)
				edit.box:SetAutoFocus(false)
				edit.box:SetText(LFMonitorDB.responses[edit.db])
				edit.box:SetScript("OnEditFocusGained", function(self) LFMonitor.EditFocus = self end)
				edit.box:SetScript("OnEditFocusLost", function() LFMonitor.EditFocus = nil end)
				edit.text:SetPoint("LEFT", edit.box, "TOPLEFT", 0, 5)
				edit.text:SetText(edit.title)
				edit.box:Show()
				anchor = edit.box

		end

		if (LFMonitorDB.monitoring.lfm == 1) then

			local function lfgMouse(self, button)

				local cursor = self:GetCursorPosition()
				local text = self:GetText()
				local segment = string.sub(text, 1, cursor)
				local link, txt, sp
				local ls, le = string.find(segment, "|c.-|h|r")

				if ((cursor == nil) or (cursor >= string.len(text))) then

					return

				end

				--remove any leading links and find the one we want

				sp = ls or 1

				while (le) do

					segment = string.sub(segment, le +1)
					sp = sp + le
					ls, le = string.find(segment, "|c.-|h|r")

				end

				ls, le = string.find(segment, "|c")

				if (ls) then

					txt = string.match(text, "|c.-|h|r", sp)
					link = string.match(text, "|H(.-)|h", sp)
					SetItemRef(link, txt, button)

				end

			end

			lfgedit:SetPoint("LEFT", anchor, "TOPLEFT", offsetx, offsety)
			lfgedit:SetWidth(385)
			lfgedit:SetHeight(32)
			lfgedit:SetMaxLetters(90)
			lfgedit:SetAutoFocus(false)
			lfgedit:SetScript("OnEditFocusGained", function(self) LFMonitor.EditFocus = self end)
			lfgedit:SetScript("OnEditFocusLost", function() LFMonitor.EditFocus = nil end)
			lfgedit:SetScript("OnMouseUp", lfgMouse)

			if (LFMonitorDB.responses.lfm == nil) then

				LFMonitorDB.responses.lfm = LFMonitor.LFG_RESPONSE

			end

			lfgedit:SetText(LFMonitorDB.responses.lfm)
			lfgtext:SetPoint("LEFT", lfgedit, "TOPLEFT", 0, 5)
			lfgtext:SetText(_G.LFG_TITLE)
			lfgedit:Show()

		else

			lfgedit:Hide()

		end

		first = true

	end

	local function responseOnHide()

		for _, edit in ipairs(edits) do

			edit.box:ClearAllPoints()
			edit.text:ClearAllPoints()

		end

	end

	frame:SetScript("OnShow", responseOnShow)
	frame:SetScript("OnHide", responseOnHide)

	return frame

end

local function CreateCustomPanel()

	local frame = CreateFrame("Frame", "LFMonitorOptionsCustom")
	local basicedit = CreateFrame("EditBox", "LFMonitorBasicEdit", frame, "InputBoxTemplate")
	local advedit = CreateFrame("EditBox", "LFMonitorAdvEdit", frame, "InputBoxTemplate")
	local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	local basictext = basicedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local advtext = advedit:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local inst = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local warn = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cchantext = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	LFMonitor.cc1 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc1text = LFMonitor.cc1:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc2 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc2text = LFMonitor.cc2:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc3 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc3text = LFMonitor.cc3:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc4 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc4text = LFMonitor.cc4:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc5 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc5text = LFMonitor.cc5:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc6 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc6text = LFMonitor.cc6:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	LFMonitor.cc7 = CreateFrame("CheckButton", nil, frame, "OptionsCheckButtonTemplate")
	LFMonitor.cc7text = LFMonitor.cc7:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	local ctl

	local function customOnShow()

		local i, v
		local channels = {GetChannelList()}

		v = 1

		for i = 1, 7 do

			LFMonitor["cc" .. i]:Hide()

		end

		for i = 1, #channels, 2 do

			if (channels[i] > 4) then

				LFMonitor["cc" .. v .. "text"]:SetText(channels[i + 1])
				LFMonitor["cc" .. v]:SetChecked(LFMonitorDB.channels["cc" .. channels[i + 1]])
				LFMonitor["cc" .. v]:Show()
				v = v + 1

			end

		end

		if (v == 1) then

			LFMonitor.cchantext:Hide()

		else

			LFMonitor.cchantext:Show()

		end

	end

	frame:Hide()
	frame:SetScript("OnShow", customOnShow)

	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -20)
	title:SetWidth(400)
	title:SetText(LFMonitor.CUSTOM_OPTIONS)

	inst:SetPoint("TOPLEFT", title, "TOPLEFT", 0, -20)
	inst:SetWidth(400)
	inst:SetText(LFMonitor.CUSTOM_INST)

	basictext:SetPoint("LEFT", inst, "TOPLEFT", 5, -30)
	basictext:SetText(LFMonitor.CUSTOM_BASIC)

	basicedit:SetPoint("LEFT", basictext, "TOPLEFT", 5, -25)
	basicedit:SetWidth(385)
	basicedit:SetHeight(32)
	basicedit:SetMaxLetters(90)
	basicedit:SetAutoFocus(false)
	basicedit:SetText(LFMonitorDB.options.basic or "")

	warn:SetPoint("TOPLEFT", basicedit, "TOPLEFT", -5, -45)
	warn:SetTextColor(1, 1, 1, 1)
	warn:SetWidth(390)
	warn:SetJustifyH("LEFT")
	warn:SetText(LFMonitor.CUSTOM_WARNING)

	advtext:SetPoint("LEFT", warn, "BOTTOMLEFT", 0, -20)
	advtext:SetText(LFMonitor.CUSTOM_ADV)

	advedit:SetPoint("LEFT", advtext, "TOPLEFT", 5, -25)
	advedit:SetWidth(385)
	advedit:SetHeight(32)
	advedit:SetMaxLetters(90)
	advedit:SetAutoFocus(false)
	advedit:SetText(LFMonitorDB.options.adv or "")

	LFMonitor.cchantext:SetPoint("TOPLEFT", advedit, "TOPLEFT", 0, -35)
	LFMonitor.cchantext:SetWidth(400)
	LFMonitor.cchantext:SetText(LFMonitor.CUSTOM_CHANNELS)

	local function ctlOnClick(idx)

		LFMonitorDB.channels["cc" .. LFMonitor["cc" .. idx .. "text"]:GetText()] = LFMonitor["cc" .. idx]:GetChecked()

	end

	for ctl = 1,7 do

		if (ctl == 1) then

			LFMonitor["cc1"]:SetPoint("TOPLEFT", LFMonitor.cchantext, "BOTTOMLEFT", 5, -5)

		else

			LFMonitor["cc" .. ctl]:SetPoint("TOPLEFT", LFMonitor["cc" .. (ctl - 1)], "BOTTOMLEFT", 0, 0)

		end

		LFMonitor["cc" .. ctl .. "text"]:SetPoint("TOPLEFT", LFMonitor["cc" .. ctl], "TOPRIGHT", 5, -5)
		LFMonitor["cc" .. ctl]:SetScript("OnClick", function() ctlOnClick(ctl) end)

	end

	return frame

end
function LFMonitor:CreateOptions()

	LFMonitor:CheckSettingsVersion(true)

	local profslist = {
		["jc"] = bi["Jewelcrafting"],
		["enc"] = bi["Enchanting"],
		["lw"] = bi["Leatherworking"],
		["ins"] = bi["Inscription"],
		["tlr"] = bi["Tailoring"],
		["bs"] = bi["Blacksmithing"],
		["alc"] = bi["Alchemy"],
		["pm"] = GetSpellInfo(28675), --Potion Master
		["em"] = GetSpellInfo(28677), --Elixir Master
		["xm"] = GetSpellInfo(28672), --Transmutation Master
		["eng"] = bi["Engineering"],
		["sm"] = GetSpellInfo(2656), --Smelting
	}

	local serviceslist = {
		["co"] = bi["Cooking"],
		["lp"] = GetSpellInfo(6460), --Lockpicking
		["port"] = loc["Mage Portals"],
		["hgwt"] = GetSpellLink(83967), -- Have Group, Will Travel
		["fly"] = loc["Mount Rides"],
		["misc"] = _G.MISCELLANEOUS,
	}

	local rolelist = {
		["tank"] = loc["Tank / Flag Carrier"],
		["healer"] = _G.HEALER,
		["melee"] = _G.PLAYERSTAT_MELEE_COMBAT,
		["ranged"] = _G.PLAYERSTAT_RANGED_COMBAT,
		["cc"] = loc["Crowd Control"]
	}

	local arenalist = {
		["2s"]  = _G.ARENA_2V2,
		["3s"]  = _G.ARENA_3V3,
		["5s"]  = _G.ARENA_5V5,
	}
	local bglist = {
		["RBG"] = _G.PVP_RATED_BATTLEGROUNDS,
	}
	local dunglist = {
		["ZA"] = bz["Zul'Aman"],
		["ZG"] = bz["Zul'Gurub"]
	}
	local catalist = {
		["FL"]	= bz["Firelands"],
		["BH"]	= bz["Baradin Hold"],
		["BWD"] = bz["Blackwing Descent"],
		["BoT"]	= bz["The Bastion of Twilight"],
		["TFW"]	= bz["Throne of the Four Winds"],
	}
	local wrathlist = {
		["EoE"] = bz["The Eye of Eternity"],
		["ICC"] = bz["Icecrown Citadel"],
		["Nax"] = bz["Naxxramas"],
		["Ony"] = bz["Onyxia's Lair"],
		["OS"] = bz["The Obsidian Sanctum"],
		["RS"] = bz["The Ruby Sanctum"],
		["ToC"] = bz["Trial of the Crusader"],
		["Uld"] = bz["Ulduar"],
		["VoA"] = bz["Vault of Archavon"],
	}
	local bclist = {
		["Hyj"] = bz["Mount Hyjal"],
		["BT"] = bz["Black Temple"],
		["GL"] = bz["Gruul's Lair"],
		["Kra"] = bz["Karazhan"],
		["Mag"] = bz["Magtheridon's Lair"],
		["SSC"] = bz["Serpentshrine Cavern"],
		["SWP"] = bz["Sunwell Plateau"],
		["TK"] = bz["Tempest Keep"],
	}
	local classiclist = {
		["BWL"] = bz["Blackwing Lair"],
		["MC"] = bz["Molten Core"],
		["AQ10"] = bz["Ruins of Ahn'Qiraj"],
		["AQ40"] = bz["Temple of Ahn'Qiraj"],
		["UBRS"] = bz["Upper Blackrock Spire"],
	}

	local function raidsArgs(type)
		local function getter(_,key)
			return LFMonitorDB[type].raids[key] or false
		end
		local function setter(_,key,val)
			LFMonitorDB[type].raids[key] = val
		end

		return {instances={
			name = loc["Instances"],
			type = "group",
			order = -1,
			args = {
				override={
					name	= loc["Don't filter by instance"],
					type	= "toggle",
					get		= function() return LFMonitorDB[type].overrideraid end,
					set		= function(_,val) LFMonitorDB[type].overrideraid = val end,
				},
				pvp = {
					width	= "double",
					name	= _G.PLAYER_V_PLAYER,
					type	= "group",
					inline	= false,
					order	= 0,
					args	= {
						arenas = {
							name	= _G.ARENA_TEAMS,
							type	= "multiselect",
							values	= arenalist,
							get		= getter,
							set		= setter,
						},
						bgs = {
							name	= _G.BATTLEFIELDS,
							type	= "multiselect",
							values	= bglist,
							get		= getter,
							set		= setter,
						}
					}
				},
				raids = {
					width	= "double",
					name	= loc["Raids"],
					type	= "group",
					inline	= false,
					order	= 1,
					args	= {
						cata = {
							width	= "double",
							name	= _G.EXPANSION_NAME3,
							type	= "multiselect",
							values	= catalist,
							get		= getter,
							set		= setter,
							order	= 97,
						},
						wrath = {
							width	= "double",
							name	= _G.EXPANSION_NAME2,
							type	= "multiselect",
							values	= wrathlist,
							get		= getter,
							set		= setter,
							order	= 98,
						},
						bc = {
							width	= "double",
							name	= _G.EXPANSION_NAME1,
							type	= "multiselect",
							values	= bclist,
							get		= getter,
							set		= setter,
							order	= 99,
						},
						classic = {
							width	= "double",
							name	= _G.EXPANSION_NAME0,
							type	= "multiselect",
							values	= classiclist,
							get		= getter,
							set		= setter,
							order	= 100,
						},
					},
				},
				dungeons = {
					width	= "double",
					name	= loc["Dungeons"],
					type	= "group",
					inline	= false,
					order	= 2,
					args	= {
						cata = {
							name	= _G.EXPANSION_NAME3,
							width	= "double",
							type	= "multiselect",
							values	= dunglist,
							get		= getter,
							set		= setter,
							order	= 2,
						}
					}
				},
			},
		}}
	end

	local function profsArgs(type)
		local function getter(_,key)
			return LFMonitorDB[type].professions[key] or false
		end
		local function setter(_,key,val)
			LFMonitorDB[type].professions[key] = val
		end
		return {
				profs={
					name	= _G.TRADE_SKILLS,
					type 	= "multiselect",
					values	= profslist,
					get		= getter,
					set		= setter,
					order	= 10,
				},
				services={
					name	= loc["Other Services"],
					type 	= "multiselect",
					values	= serviceslist,
					get		= getter,
					set		= setter,
					order	= 11,
				}
		}
	end

	local function classRoleArgs(type)
		return {classroles={
			name	= loc["Classes"] .. "/" .. loc["Roles"],
			type	= "group",
			order	= 20,
			args	= {
				roles={
					name 	= loc["Roles"],
					type 	= "multiselect",
					values 	= rolelist,
					get		= function(_,key) return LFMonitorDB[type].roles[key] or false end,
					set		= function(_,key, val) LFMonitorDB[type].roles[key] = val end,
					order	= 30,
				},
				classes={
					name 	= loc["Classes"],
					type 	= "multiselect",
					values 	= _G.LOCALIZED_CLASS_NAMES_MALE,
					get		= function(_,key) return LFMonitorDB[type].classes[key] or false end,
					set		= function(_,key, val) LFMonitorDB[type].classes[key] = val end,
					order	= 20,
				},
			}
		}}
	end

	local function createTab(type, longname, extrafuncs, inline)
		local tab = {
			type	= "group",
			inline	= inline or false,
			childGroups = "tree",
			args    = {
				header = {
					name   = longname,
					type   = "header",
					order  = 0,
					hidden = inline or false
				},
				enable = {
					name	= _G.ENABLE,
					desc	= loc["Search for and display messages of this type"],
					type	= "toggle",
					order   = 1,
					get		= function () return LFMonitorDB.monitoring[type] or false end,
					set		= function (_,val) LFMonitorDB.monitoring[type] = val end,
				}
			},
			plugins = {}
		}
		if inline then
			tab.name	= longname
		else
			tab.name	= LFMonitor.ABBRS.type[type]
			tab.desc	= longname
		end
		local k,v
		for k,v in ipairs(extrafuncs) do
			tab.plugins[""..k] = v(type)
		end
		return tab
	end

	LFMonitor.FilterOptionTable = {
	  name			= loc["Filtering"],
	  type			= "group",
	  childGroups	= "tab",
	  args 			= {
		lfw		= createTab("lfw",		loc["Looking for Work"],			{profsArgs}),
		lfprof	= createTab("lfprof",	loc["Looking for Crafter/Service"], {profsArgs}),
		lfg		= createTab("lfg",		_G.LFG_TITLE,						{classRoleArgs,raidsArgs}),
		lfm		= createTab("lfm",		_G.LFM_TITLE,						{classRoleArgs,raidsArgs}),
		lfguild	= createTab("lfguild",	loc["Looking for Guild"],			{classRoleArgs}),
		guildlfm= createTab("guildlfm",	loc["Guild Recruitment"],			{classRoleArgs}),
		wtbwts  = {
			name	= LFMonitor.ABBRS.type["wtb"] .. " / " .. LFMonitor.ABBRS.type["wts"],
			desc	= loc["Want to Buy"] .. " / " .. loc["Want to Sell"],
			type	= "group",
			args	= {
				wtb = createTab("wtb", loc["Want to Buy"], {}, true),
				wts = createTab("wts", loc["Want to Sell"], {}, true),
			}
		}
	  }
	}

	local mainPanel = CreateMainPanel()
	local responsePanel = CreateResponsePanel()
	local generalPanel = CreateGeneralPanel()
	local appearancePanel = CreateAppearancePanel()
	local customPanel = CreateCustomPanel()

	mainPanel.name = "LFMonitor"
	mainPanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(mainPanel)

	generalPanel.name = _G.GENERAL_LABEL
	generalPanel.parent = "LFMonitor"
	generalPanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(generalPanel)

	appearancePanel.name = _G.APPEARANCE_LABEL
	appearancePanel.parent = "LFMonitor"
	appearancePanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(appearancePanel)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("LFMonitor Filtering", LFMonitor.FilterOptionTable)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("LFMonitor Filtering", loc["Filtering"], "LFMonitor")

	responsePanel.name = LFMonitor.RESPONSES
	responsePanel.parent = "LFMonitor"
	responsePanel.okay = function() LFMonitor:ResponseOptionsOkay() end
	responsePanel.cancel = function() LFMonitor:BuildTables() end
	responsePanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	InterfaceOptions_AddCategory(responsePanel)

	customPanel.name = LFMonitor.CUSTOM_OPTIONS
	customPanel.parent = "LFMonitor"
	customPanel.default = function() StaticPopup_Show("LFMONITOR_DEFAULTS") end
	customPanel.okay = function() LFMonitor:CustomOptionsOkay() end

	InterfaceOptions_AddCategory(customPanel)

end
